#include <stdio.h>
#include <erl_driver.h>

#define INIT_INT64_LEN 1024
#define INIT_TERM_LEN 4096

int
test_decode(ErlDrvPort port, char* buf, int len, char** rbuf, int rlen)
{
	long long signed int testvalue=0;
	//fprintf(stderr, "buf: [%-10.20s]\n", buf);
    testvalue = strtoll((const char *) buf, NULL, 10);
	//fprintf(stderr, "testvalue: %lld\n", testvalue);
			

	//buffer to store the terms
    ErlDrvTermData*  	term_data;
    int 				term_length;
    int 				term_used;
	int 				resp;

	//temp buffer for my int64 data
    ErlDrvSInt64*  	int64_data;
    int             int64_length;
    int             int64_used;

	//init term-data
    term_data = (ErlDrvTermData*) malloc(INIT_TERM_LEN * sizeof(ErlDrvTermData));
    if(term_data == NULL) goto done;
    term_length = INIT_TERM_LEN;
    term_used = 0;

	//init int64-data
    int64_data = (ErlDrvSInt64*) malloc(INIT_INT64_LEN * sizeof(ErlDrvSInt64));
    if(int64_data == NULL) goto done;
    int64_length = INIT_INT64_LEN;
    int64_used = 0;

    int ret = -1;

	//add a large integer to the output
	term_data[term_used++] = ERL_DRV_INT64;
	int64_data[int64_used++] = (ErlDrvSInt64) testvalue;
	ErlDrvSInt64* pos = int64_data + (int64_used-1);
	term_data[term_used++] = (ErlDrvTermData) pos;

	//sanity check
	//fprintf(stderr, "testvalue: %lld, pos: %ld, *pos: %lld\n", testvalue, pos, *pos);

    *rbuf = NULL;
    resp = driver_send_term(
        port,
        driver_caller(port),
        term_data,
        term_used
    );
    if(resp == 1) ret = 0;

done:
	if(term_data != NULL) free(term_data);
	if(int64_data != NULL) free(int64_data);
	
    return ret;
}


static ErlDrvData
bignum_start(ErlDrvPort port, char *buff)
{
    if(port == NULL) return ERL_DRV_ERROR_GENERAL;
    set_port_control_flags(port, PORT_CONTROL_FLAG_BINARY);
    return (ErlDrvData) port;
}

static int
bignum_control(
        ErlDrvData drv_data,
        unsigned int command,
        char* buf,
        int len,
        char** rbuf,
        int rlen
)
{
    switch(command)
    {
        case 0:		
			//fprintf(stderr, "test_decode\n");
            return test_decode((ErlDrvPort) drv_data, buf, len, rbuf, rlen);
        default:
            return -1;
    }
}

static ErlDrvEntry
bignum_driver_entry =
{
    NULL,               /* Init */
    bignum_start,
    NULL,               /* Stop */
    NULL,               /* Output */
    NULL,               /* Input Ready */
    NULL,               /* Output Ready */
    "bignum_drv",      /* Driver Name */
    NULL,               /* Finish */
    NULL,               /* Handle */
    bignum_control,    /* Control */
    NULL,               /* Timeout */
    NULL,               /* Outputv */
    NULL,               /* Ready Async */
    NULL,               /* Flush */
    NULL,               /* Call */
    NULL,               /* Event */
    ERL_DRV_EXTENDED_MARKER,
    ERL_DRV_EXTENDED_MAJOR_VERSION,
    ERL_DRV_EXTENDED_MINOR_VERSION,
    ERL_DRV_FLAG_USE_PORT_LOCKING,
    NULL,               /* Reserved */
    NULL,               /* Process Exit */
};

DRIVER_INIT(bignum_drv)    /* must match name in driver_entry */
{
    return &bignum_driver_entry;
}

